#define F_CPU 16000000 
#include <stdint.h>
#include <math.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAX_MEOW_DISTANCE 10
#define HC_SR04_NUM_DEVICES	1

#define ECHO_PIN PA1
#define TRIGGER_PIN PA0
#define SERVO_PIN PB4


static volatile uint8_t flag = 0;
static volatile uint8_t count = 0;
static volatile uint8_t saved_tcnt0 = 0;
static volatile uint8_t saved_count = 0;

//Bluetooth code
void USART_init()
{
	UBRR0H = 0;
	UBRR0L = 103;

	// start the transmision
	UCSR0B = (1<<TXEN0) | (1<<RXEN0);

	 // set up the framework
	 UCSR0C = (3<<UCSZ00);
}

void USART_transmit(char data)
{
    while(!(UCSR0A & (1<<UDRE0))) ;
    
    //put data in the buffer
    UDR0 = data;
}


//Transmit a buffer
void USART_stransmit(char *data) 
{
	while(*data != '\0')
		USART_transmit(*data++);
}


//Timer code
ISR(TIMER0_OVF_vect, ISR_NOBLOCK)
{
	count++;

}

ISR(PCINT0_vect, ISR_NOBLOCK)
{
	if ((PINA & (1 << ECHO_PIN)) != 0) {
	// Echo rising edge
		TCNT0 = 0;  
		count = 0;
	}
	else {
	// Echo falling edge
		saved_tcnt0 = TCNT0;
		saved_count = count;
		flag = 1;
	}
}

//Ultrasonic sensor code
void HC_SR04_init()
{
	TCCR0A = 0;
	TCCR0B = 0;
	TIMSK0 = 0;

	//E nable global interrupts
	sei(); 

	// Prescaler = 256 => T = 16us
	TCCR0B |= (1 << CS02); 
	// Enable Overflow interrupt
	TIMSK0 |= (1 << TOIE0); 

	// Configure pins for trigger and echo
	DDRA |= (1 << TRIGGER_PIN); 	
	DDRA &= ~(1 << ECHO_PIN); 

	PORTA &= ~(1 << TRIGGER_PIN);


	/* Enable pin change interrupt on echo pin */
	PCICR |= (1 << PCIE0);
	PCMSK0 = 0;
	PCMSK0 |= (1 << PCINT1);

}

double HC_SR04_get_distance()
{
	double sum = 0;
	flag = 0;

	// Reset Trigger signal for a clean rise
	PORTA &= ~(1 << TRIGGER_PIN);
	_delay_us(5);

	// Send Trigger signal for > 10us
	PORTA |= (1 << TRIGGER_PIN);
	_delay_us(11);
	PORTA &= ~(1 << TRIGGER_PIN);

	while(flag == 0 && count < 5);

	sum = sum + (((int)saved_tcnt0 + 255 * saved_count) * 16.0) * 0.017;
	
	return sum;
}

//Servomotor code
void servo_rotate() { 
	
	for ( int j = 0; j < 1; j++) {
		int n = 1;
		while(n <= 23) { 
			PORTB |= (1 << SERVO_PIN); 
			for(int i = 0; i < n; i++){
				_delay_us(50); 
			}
 
			PORTB &= ~(1 << SERVO_PIN); 
			_delay_ms(100); 
			n++;
		} 

		PORTB |= (1 << PB4); 

		for(int i = 0; i < 3; i++){
			_delay_us(75); 
		}
 
		PORTB &= ~(1 << SERVO_PIN); 
		_delay_ms(120); 

		PORTB |= (1 << SERVO_PIN); 
		for(int i = 0; i < 3; i++){

			 _delay_us(72);

		}
 
		PORTB &= ~(1 << SERVO_PIN);
		_delay_ms(120);

		PORTB |= (1 << SERVO_PIN); 
		for(int i = 0; i < 3; i++){

			 _delay_us(80);

		} 
		PORTB &= ~(1 << SERVO_PIN);  
	}	
 }

//Main programm
int main() {

	//Init the system
	char buffer[40];
	USART_init();
	HC_SR04_init();

	DDRB |= (1 << SERVO_PIN);

	while(1) {
		
		//Reading the code
		double dist = HC_SR04_get_distance();
		_delay_ms(5);
	
		
		if(dist < MAX_MEOW_DISTANCE ) {

			buffer[0] = '\0';
			strcpy(buffer, "The cat has just been fed\n");
			_delay_ms(10);

			USART_stransmit(buffer);
			_delay_ms(10);

			servo_rotate();
			_delay_ms(20);
						
		}
		 

		_delay_ms(20);
	}
	return 0;
}
